<?php
namespace EEPROM;

define('PRM_STARTUP', microtime(true));
define('PRM_ROOT', __DIR__);
define('PRM_CLI', PHP_SAPI === 'cli');
define('PRM_DEBUG', is_file(PRM_ROOT . '/.debug'));
define('PRM_PHP_MIN_VER', '7.4.0');
define('PRM_PUBLIC', PRM_ROOT . '/public');
define('PRM_SOURCE', PRM_ROOT . '/src');
define('PRM_UPLOADS', PRM_PUBLIC . '/data');
define('PRM_THUMBS', PRM_PUBLIC . '/thumb');

if(version_compare(PHP_VERSION, PRM_PHP_MIN_VER, '<'))
    die('EEPROM required at least PHP ' . PRM_PHP_MIN_VER . '.');

error_reporting(PRM_DEBUG ? -1 : 0);
ini_set('display_errors', PRM_DEBUG ? 'On' : 'Off');

mb_internal_encoding('utf-8');
date_default_timezone_set('utc');

set_exception_handler(function(\Throwable $ex) {
    http_response_code(500);
    header('Content-Type: text/plain; charset=utf-8');
    if(PRM_DEBUG)
        echo (string)$ex;
    exit;
});

set_error_handler(function(int $errno, string $errstr, string $errfile, int $errline) {
    throw new \ErrorException($errstr, 0, $errno, $errfile, $errline);
    return true;
}, -1);

spl_autoload_register(function(string $className) {
    if(substr($className, 0, 7) !== 'EEPROM\\')
        return;

    $classPath = PRM_SOURCE . str_replace('\\', '/', substr($className, 6)) . '.php';

    if(is_file($classPath))
        require_once $classPath;
});

if(!is_dir(PRM_UPLOADS))
    mkdir(PRM_UPLOADS, 0775, true);
if(!is_dir(PRM_THUMBS))
    mkdir(PRM_THUMBS, 0775, true);

$configPath = PRM_ROOT . '/config.ini';

if(!is_file($configPath))
    die('EEPROM configuration is missing.');

Config::load($configPath);

if(!Config::has('PDO', 'dsn') || !Config::has('PDO', 'username'))
    die('EEPROM database is not configured.');

DB::init(
    Config::get('PDO', 'dsn'),
    Config::get('PDO', 'username'),
    Config::get('PDO', 'password', ''),
    DB::FLAGS
);
